@@ -1,4 +1,5 @@
 # Copyright © 2007-2009 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2012-2024 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,10 +28,9 @@ CTRL_* constants exported by L<Dpkg::Control>.
 
 =cut
 
-package Dpkg::Control::FieldsCore 1.02;
+package Dpkg::Control::FieldsCore 1.04;
 
-use strict;
-use warnings;
+use v5.36;
 
 our @EXPORT = qw(
     field_capitalize
@@ -43,6 +43,7 @@ our @EXPORT = qw(
     field_list_pkg_dep
     field_get_dep_type
     field_get_sep_type
+    field_get_default_value
     field_ordered_list
     field_register
     field_insert_after
@@ -60,8 +61,8 @@ use Dpkg::ErrorHandling;
 use Dpkg::Control::Types;
 
 use constant {
-    ALL_PKG => CTRL_INFO_PKG | CTRL_INDEX_PKG | CTRL_PKG_DEB | CTRL_FILE_STATUS,
-    ALL_SRC => CTRL_INFO_SRC | CTRL_INDEX_SRC | CTRL_PKG_SRC,
+    ALL_PKG => CTRL_TMPL_PKG | CTRL_REPO_PKG | CTRL_DEB | CTRL_FILE_STATUS,
+    ALL_SRC => CTRL_TMPL_SRC | CTRL_REPO_SRC | CTRL_DSC,
     ALL_FILE_MANIFEST => CTRL_FILE_BUILDINFO | CTRL_FILE_CHANGES,
     ALL_CHANGES => CTRL_FILE_CHANGES | CTRL_CHANGELOG,
     ALL_COPYRIGHT => CTRL_COPYRIGHT_HEADER | CTRL_COPYRIGHT_FILES | CTRL_COPYRIGHT_LICENSE,
@@ -85,7 +86,7 @@ our %FIELDS = (
     },
     'architecture' => {
         name => 'Architecture',
-        allowed => (ALL_PKG | ALL_SRC | ALL_FILE_MANIFEST | CTRL_TESTS) & (~CTRL_INFO_SRC),
+        allowed => (ALL_PKG | ALL_SRC | ALL_FILE_MANIFEST | CTRL_TESTS) & (~CTRL_TMPL_SRC),
         separator => FIELD_SEP_SPACE,
     },
     'architectures' => {
@@ -95,12 +96,12 @@ our %FIELDS = (
     },
     'auto-built-package' => {
         name => 'Auto-Built-Package',
-        allowed => ALL_PKG & ~CTRL_INFO_PKG,
+        allowed => ALL_PKG & ~CTRL_TMPL_PKG,
         separator => FIELD_SEP_SPACE,
     },
     'binary' => {
         name => 'Binary',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | ALL_FILE_MANIFEST,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | ALL_FILE_MANIFEST,
         # XXX: This field values are separated either by space or comma
         # depending on the context.
         separator => FIELD_SEP_SPACE | FIELD_SEP_COMMA,
@@ -122,7 +123,7 @@ our %FIELDS = (
     },
     'bugs' => {
         name => 'Bugs',
-        allowed => (ALL_PKG | CTRL_INFO_SRC | CTRL_FILE_VENDOR) & (~CTRL_INFO_PKG),
+        allowed => (ALL_PKG | CTRL_TMPL_SRC | CTRL_FILE_VENDOR) & (~CTRL_TMPL_PKG),
     },
     'build-architecture' => {
         name => 'Build-Architecture',
@@ -174,6 +175,10 @@ our %FIELDS = (
         dependency => 'normal',
         dep_order => 3,
     },
+    'build-driver' => {
+        name => 'Build-Driver',
+        allowed => CTRL_TMPL_SRC,
+    },
     'build-essential' => {
         name => 'Build-Essential',
         allowed => ALL_PKG,
@@ -192,7 +197,7 @@ our %FIELDS = (
     },
     'build-profiles' => {
         name => 'Build-Profiles',
-        allowed => CTRL_INFO_PKG,
+        allowed => CTRL_TMPL_PKG,
         separator => FIELD_SEP_SPACE,
     },
     'build-tainted-by' => {
@@ -230,15 +235,15 @@ our %FIELDS = (
     },
     'checksums-md5' => {
         name => 'Checksums-Md5',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | ALL_FILE_MANIFEST,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | ALL_FILE_MANIFEST,
     },
     'checksums-sha1' => {
         name => 'Checksums-Sha1',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | ALL_FILE_MANIFEST,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | ALL_FILE_MANIFEST,
     },
     'checksums-sha256' => {
         name => 'Checksums-Sha256',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | ALL_FILE_MANIFEST,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | ALL_FILE_MANIFEST,
     },
     'classes' => {
         name => 'Classes',
@@ -304,7 +309,7 @@ our %FIELDS = (
     },
     'directory' => {
         name => 'Directory',
-        allowed => CTRL_INDEX_SRC,
+        allowed => CTRL_REPO_SRC,
     },
     'distribution' => {
         name => 'Distribution',
@@ -333,17 +338,17 @@ our %FIELDS = (
     },
     'filename' => {
         name => 'Filename',
-        allowed => CTRL_INDEX_PKG,
+        allowed => CTRL_REPO_PKG,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'files' => {
         name => 'Files',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | CTRL_FILE_CHANGES | CTRL_COPYRIGHT_FILES,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | CTRL_FILE_CHANGES | CTRL_COPYRIGHT_FILES,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'format' => {
         name => 'Format',
-        allowed => CTRL_PKG_SRC | CTRL_INDEX_SRC | ALL_FILE_MANIFEST | CTRL_COPYRIGHT_HEADER,
+        allowed => CTRL_DSC | CTRL_REPO_SRC | ALL_FILE_MANIFEST | CTRL_COPYRIGHT_HEADER,
     },
     'homepage' => {
         name => 'Homepage',
@@ -358,7 +363,7 @@ our %FIELDS = (
     },
     'installed-size' => {
         name => 'Installed-Size',
-        allowed => ALL_PKG & ~CTRL_INFO_PKG,
+        allowed => ALL_PKG & ~CTRL_TMPL_PKG,
     },
     'installer-menu-item' => {
         name => 'Installer-Menu-Item',
@@ -378,16 +383,16 @@ our %FIELDS = (
     },
     'origin' => {
         name => 'Origin',
-        allowed => (ALL_PKG | ALL_SRC | CTRL_REPO_RELEASE) & (~CTRL_INFO_PKG),
+        allowed => (ALL_PKG | ALL_SRC | CTRL_REPO_RELEASE) & (~CTRL_TMPL_PKG),
     },
     'maintainer' => {
         name => 'Maintainer',
-        allowed => CTRL_PKG_DEB | CTRL_INDEX_PKG | CTRL_FILE_STATUS | ALL_SRC  | ALL_CHANGES,
+        allowed => CTRL_DEB | CTRL_REPO_PKG | CTRL_FILE_STATUS | ALL_SRC  | ALL_CHANGES,
     },
     'md5sum' => {
         # XXX: Wrong capitalization due to historical reasons.
         name => 'MD5sum',
-        allowed => CTRL_INDEX_PKG | CTRL_REPO_RELEASE,
+        allowed => CTRL_REPO_PKG | CTRL_REPO_RELEASE,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'multi-arch' => {
@@ -404,11 +409,11 @@ our %FIELDS = (
     },
     'package' => {
         name => 'Package',
-        allowed => ALL_PKG | CTRL_INDEX_SRC,
+        allowed => ALL_PKG | CTRL_REPO_SRC,
     },
     'package-list' => {
         name => 'Package-List',
-        allowed => ALL_SRC & ~CTRL_INFO_SRC,
+        allowed => ALL_SRC & ~CTRL_TMPL_SRC,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'package-type' => {
@@ -428,7 +433,8 @@ our %FIELDS = (
     },
     'priority' => {
         name => 'Priority',
-        allowed => CTRL_INFO_SRC | CTRL_INDEX_SRC | ALL_PKG,
+        allowed => CTRL_TMPL_SRC | CTRL_REPO_SRC | ALL_PKG,
+        default => 'optional',
     },
     'protected' => {
         name => 'Protected',
@@ -462,34 +468,35 @@ our %FIELDS = (
     },
     'rules-requires-root' => {
         name => 'Rules-Requires-Root',
-        allowed => CTRL_INFO_SRC,
+        allowed => CTRL_TMPL_SRC,
         separator => FIELD_SEP_SPACE,
     },
     'section' => {
         name => 'Section',
-        allowed => CTRL_INFO_SRC | CTRL_INDEX_SRC | ALL_PKG,
+        allowed => CTRL_TMPL_SRC | CTRL_REPO_SRC | ALL_PKG,
+        default => 'unknown',
     },
     'sha1' => {
         # XXX: Wrong capitalization due to historical reasons.
         name => 'SHA1',
-        allowed => CTRL_INDEX_PKG | CTRL_REPO_RELEASE,
+        allowed => CTRL_REPO_PKG | CTRL_REPO_RELEASE,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'sha256' => {
         # XXX: Wrong capitalization due to historical reasons.
         name => 'SHA256',
-        allowed => CTRL_INDEX_PKG | CTRL_REPO_RELEASE,
+        allowed => CTRL_REPO_PKG | CTRL_REPO_RELEASE,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'size' => {
         name => 'Size',
-        allowed => CTRL_INDEX_PKG,
+        allowed => CTRL_REPO_PKG,
         separator => FIELD_SEP_LINE | FIELD_SEP_SPACE,
     },
     'source' => {
         name => 'Source',
         allowed => (ALL_PKG | ALL_SRC | ALL_CHANGES | CTRL_COPYRIGHT_HEADER | CTRL_FILE_BUILDINFO) &
-                   (~(CTRL_INDEX_SRC | CTRL_INFO_PKG)),
+                   (~(CTRL_REPO_SRC | CTRL_TMPL_PKG)),
     },
     'standards-version' => {
         name => 'Standards-Version',
@@ -636,7 +643,7 @@ our %FIELDS = (
     'version' => {
         name => 'Version',
         allowed => (ALL_PKG | ALL_SRC | CTRL_FILE_BUILDINFO | ALL_CHANGES | CTRL_REPO_RELEASE) &
-                    (~(CTRL_INFO_SRC | CTRL_INFO_PKG)),
+                    (~(CTRL_TMPL_SRC | CTRL_TMPL_PKG)),
     },
 );
 
@@ -691,7 +698,7 @@ my @bin_checksums_fields = qw(
 );
 
 our %FIELD_ORDER = (
-    CTRL_INFO_SRC() => [
+    CTRL_TMPL_SRC() => [
         qw(
             source
             section
@@ -713,7 +720,7 @@ our %FIELD_ORDER = (
             description
         ),
     ],
-    CTRL_INFO_PKG() => [
+    CTRL_TMPL_PKG() => [
         qw(
             package
             package-type
@@ -738,7 +745,7 @@ our %FIELD_ORDER = (
             description
         ),
     ],
-    CTRL_PKG_SRC() => [
+    CTRL_DSC() => [
         qw(
             format
             source
@@ -763,7 +770,7 @@ our %FIELD_ORDER = (
             files
         ),
     ],
-    CTRL_PKG_DEB() => [
+    CTRL_DEB() => [
         qw(
             package
             package-type
@@ -794,7 +801,7 @@ our %FIELD_ORDER = (
             task
         ),
     ],
-    CTRL_INDEX_SRC() => [
+    CTRL_REPO_SRC() => [
         qw(
             format
             package
@@ -822,7 +829,7 @@ our %FIELD_ORDER = (
             files
         ),
     ],
-    CTRL_INDEX_PKG() => [
+    CTRL_REPO_PKG() => [
         qw(
             package
             package-type
@@ -971,7 +978,7 @@ our %FIELD_ORDER = (
         ),
     ],
     CTRL_FILE_STATUS() => [
-        # Same as fieldinfos in lib/dpkg/parse.c
+        # Same as fieldinfos in «lib/dpkg/parse.c».
         qw(
             package
             essential
@@ -1043,13 +1050,13 @@ except the first of each word (words are separated by a hyphen in field names).
 
 =cut
 
-sub field_capitalize($) {
+sub field_capitalize {
     my $field = lc(shift);
 
     # Use known fields first.
     return $FIELDS{$field}{name} if exists $FIELDS{$field};
 
-    # Generic case
+    # Generic case.
     return join '-', map { ucfirst } split /-/, $field;
 }
 
@@ -1059,7 +1066,7 @@ Returns true if the field is official and known.
 
 =cut
 
-sub field_is_official($) {
+sub field_is_official {
     my $field = lc shift;
 
     return exists $FIELDS{$field};
@@ -1078,7 +1085,7 @@ Undef is returned for non-official fields.
 
 =cut
 
-sub field_is_allowed_in($@) {
+sub field_is_allowed_in {
     my ($field, @types) = @_;
     $field = lc $field;
 
@@ -1086,7 +1093,8 @@ sub field_is_allowed_in($@) {
 
     return 0 if not scalar(@types);
     foreach my $type (@types) {
-        next if $type == CTRL_UNKNOWN; # Always allowed
+        # Unknown control type is always allowed.
+        next if $type == CTRL_UNKNOWN;
         return 0 unless $FIELDS{$field}{allowed} & $type;
     }
     return 1;
@@ -1099,12 +1107,12 @@ $from L<Dpkg::Control> object to the $to L<Dpkg::Control> object.
 
 Official fields are copied only if the field is allowed in both types of
 objects. Custom fields are treated in a specific manner. When the target
-is not among CTRL_PKG_SRC, CTRL_PKG_DEB or CTRL_FILE_CHANGES, then they
+is not among CTRL_DSC, CTRL_DEB or CTRL_FILE_CHANGES, then they
 are always copied as is (the X- prefix is kept). Otherwise they are not
 copied except if the target object matches the target destination encoded
 in the field name. The initial X denoting custom fields can be followed by
-one or more letters among "S" (Source: corresponds to CTRL_PKG_SRC), "B"
-(Binary: corresponds to CTRL_PKG_DEB) or "C" (Changes: corresponds to
+one or more letters among "S" (Source: corresponds to CTRL_DSC), "B"
+(Binary: corresponds to CTRL_DEB) or "C" (Changes: corresponds to
 CTRL_FILE_CHANGES).
 
 Returns undef if nothing has been copied or the name of the new field
@@ -1112,14 +1120,8 @@ added to $to otherwise.
 
 =cut
 
-sub field_transfer_single($$;$) {
+sub field_transfer_single {
     my ($from, $to, $field) = @_;
-    if (not defined $field) {
-        warnings::warnif('deprecated',
-            'using Dpkg::Control::Fields::field_transfer_single() with an ' .
-            'an implicit field argument is deprecated');
-        $field = $_;
-    }
     my ($from_type, $to_type) = ($from->get_type(), $to->get_type());
     $field = field_capitalize($field);
 
@@ -1128,21 +1130,27 @@ sub field_transfer_single($$;$) {
         return $field;
     } elsif ($field =~ /^X([SBC]*)-/i) {
         my $dest = $1;
-        if (($dest =~ /B/i and $to_type == CTRL_PKG_DEB) or
-            ($dest =~ /S/i and $to_type == CTRL_PKG_SRC) or
+
+        if ($dest =~ /SC/i and $from_type == CTRL_TMPL_PKG) {
+            warning(g_('non-sensical field export rules (%s) in field %s in ' .
+                       'binary stanza from source package template control file'),
+                    $dest, $field);
+        }
+        if (($dest =~ /B/i and $to_type == CTRL_DEB) or
+            ($dest =~ /S/i and $to_type == CTRL_DSC) or
             ($dest =~ /C/i and $to_type == CTRL_FILE_CHANGES))
         {
             my $new = $field;
             $new =~ s/^X([SBC]*)-//i;
             $to->{$new} = $from->{$field};
             return $new;
-        } elsif ($to_type != CTRL_PKG_DEB and
-		 $to_type != CTRL_PKG_SRC and
-		 $to_type != CTRL_FILE_CHANGES)
-	{
-	    $to->{$field} = $from->{$field};
-	    return $field;
-	}
+        } elsif ($to_type != CTRL_DEB and
+                 $to_type != CTRL_DSC and
+                 $to_type != CTRL_FILE_CHANGES)
+        {
+            $to->{$field} = $from->{$field};
+            return $field;
+        }
     } elsif (not field_is_allowed_in($field, $from_type)) {
         warning(g_("unknown information field '%s' in input data in %s"),
                 $field, $from->get_option('name') || g_('control information'));
@@ -1159,7 +1167,7 @@ Returns the list of fields that have been added to $to.
 
 =cut
 
-sub field_transfer_all($$) {
+sub field_transfer_all {
     my ($from, $to) = @_;
     my (@res, $res);
     foreach my $k (keys %$from) {
@@ -1177,7 +1185,7 @@ The list might be empty for types where the order does not matter much.
 
 =cut
 
-sub field_ordered_list($) {
+sub field_ordered_list {
     my $type = shift;
 
     if (exists $FIELD_ORDER{$type}) {
@@ -1196,16 +1204,16 @@ from the binary version.
 Returns a list with the $source name, and the source $version, or undef
 or an empty list when $ctrl does not contain a binary package control stanza.
 Neither $source nor $version are validated, but that can be done with
-Dpkg::Package::pkg_name_is_illegal() and Dpkg::Version::version_check().
+Dpkg::Package::pkg_name_is_invalid() and Dpkg::Version::version_check().
 
 =cut
 
-sub field_parse_binary_source($) {
+sub field_parse_binary_source {
     my $ctrl = shift;
     my $ctrl_type = $ctrl->get_type();
 
-    if ($ctrl_type != CTRL_INDEX_PKG and
-        $ctrl_type != CTRL_PKG_DEB and
+    if ($ctrl_type != CTRL_REPO_PKG and
+        $ctrl_type != CTRL_DEB and
         $ctrl_type != CTRL_FILE_CHANGES and
         $ctrl_type != CTRL_FILE_BUILDINFO and
         $ctrl_type != CTRL_FILE_STATUS) {
@@ -1239,13 +1247,13 @@ Debian package.
 
 =cut
 
-sub field_list_src_dep() {
+sub field_list_src_dep {
     my @list = map {
         $FIELDS{$_}{name}
     } sort {
         $FIELDS{$a}{dep_order} <=> $FIELDS{$b}{dep_order}
     } grep {
-        field_is_allowed_in($_, CTRL_PKG_SRC) and
+        field_is_allowed_in($_, CTRL_DSC) and
         exists $FIELDS{$_}{dependency}
     } keys %FIELDS;
     return @list;
@@ -1259,13 +1267,13 @@ the stronger to the weaker.
 
 =cut
 
-sub field_list_pkg_dep() {
+sub field_list_pkg_dep {
     my @list = map {
         $FIELDS{$_}{name}
     } sort {
         $FIELDS{$a}{dep_order} <=> $FIELDS{$b}{dep_order}
     } grep {
-        field_is_allowed_in($_, CTRL_PKG_DEB) and
+        field_is_allowed_in($_, CTRL_DEB) and
         exists $FIELDS{$_}{dependency}
     } keys %FIELDS;
     return @list;
@@ -1280,11 +1288,11 @@ Breaks, ...). Returns undef for fields which are not dependencies.
 
 =cut
 
-sub field_get_dep_type($) {
+sub field_get_dep_type {
     my $field = lc shift;
 
-    return unless exists $FIELDS{$field};
-    return $FIELDS{$field}{dependency} if exists $FIELDS{$field}{dependency};
+    return $FIELDS{$field}{dependency}
+        if exists $FIELDS{$field} && exists $FIELDS{$field}{dependency};
     return;
 }
 
@@ -1295,13 +1303,29 @@ FIELD_SEP_SPACE, FIELD_SEP_COMMA or FIELD_SEP_LINE.
 
 =cut
 
-sub field_get_sep_type($) {
+sub field_get_sep_type {
     my $field = lc shift;
 
-    return $FIELDS{$field}{separator} if exists $FIELDS{$field}{separator};
+    return $FIELDS{$field}{separator}
+        if exists $FIELDS{$field} && exists $FIELDS{$field}{separator};
     return FIELD_SEP_UNKNOWN;
 }
 
+=item $value = field_get_default_value($field)
+
+Return the default value (if any) for the field. If there is no default value
+then it returns "undef".
+
+=cut
+
+sub field_get_default_value {
+    my $field = lc shift;
+
+    return $FIELDS{$field}{default}
+        if exists $FIELDS{$field} && exists $FIELDS{$field}{default};
+    return;
+}
+
 =item field_register($field, $allowed_types, %opts)
 
 Register a new field as being allowed in control information of specified
@@ -1309,7 +1333,7 @@ types. %opts is optional.
 
 =cut
 
-sub field_register($$;@) {
+sub field_register {
     my ($field, $types, %opts) = @_;
 
     $field = lc $field;
@@ -1331,7 +1355,7 @@ Return true if the field was inserted, otherwise false.
 
 =cut
 
-sub field_insert_after($$@) {
+sub field_insert_after {
     my ($type, $field, @fields) = @_;
 
     return 0 if not exists $FIELD_ORDER{$type};
@@ -1353,7 +1377,7 @@ Return true if the field was inserted, otherwise false.
 
 =cut
 
-sub field_insert_before($$@) {
+sub field_insert_before {
     my ($type, $field, @fields) = @_;
 
     return 0 if not exists $FIELD_ORDER{$type};
@@ -1370,6 +1394,14 @@ sub field_insert_before($$@) {
 
 =head1 CHANGES
 
+=head2 Version 1.04 (dpkg 1.23.0)
+
+Remove argument: field_transfer_single() implicit argument usage.
+
+=head2 Version 1.03 (dpkg 1.22.12)
+
+New function: field_get_default_value().
+
 =head2 Version 1.02 (dpkg 1.22.0)
 
 Deprecate argument: field_transfer_single() implicit argument usage.
