@@ -70,8 +70,7 @@ is no restriction formula. Otherwise it is an array ref.
 
 package Dpkg::Deps::Simple 1.02;
 
-use strict;
-use warnings;
+use v5.36;
 
 use Carp;
 
@@ -89,24 +88,26 @@ use parent qw(Dpkg::Interface::Storable);
 
 =item $dep = Dpkg::Deps::Simple->new([$dep[, %opts]]);
 
-Creates a new object. Some options can be set through %opts:
+Creates a new object.
+
+Options:
 
 =over
 
-=item host_arch
+=item B<host_arch>
 
 Sets the host architecture.
 
-=item build_arch
+=item B<build_arch>
 
 Sets the build architecture.
 
-=item build_dep
+=item B<build_dep>
 
 Specifies whether the parser should consider it a build dependency.
 Defaults to 0.
 
-=item tests_dep
+=item B<tests_dep>
 
 Specifies whether the parser should consider it a tests dependency.
 Defaults to 0.
@@ -165,41 +166,43 @@ parsed dependency.
 sub parse_string {
     my ($self, $dep) = @_;
 
-    my $pkgname_re;
+    my $pkgname_regex;
     if ($self->{tests_dep}) {
-        $pkgname_re = qr/[\@a-zA-Z0-9][\@a-zA-Z0-9+.-]*/;
+        $pkgname_regex = qr/[\@a-zA-Z0-9][\@a-zA-Z0-9+.-]*/;
     } else {
-        $pkgname_re = qr/[a-zA-Z0-9][a-zA-Z0-9+.-]*/;
+        $pkgname_regex = qr/[a-zA-Z0-9][a-zA-Z0-9+.-]*/;
     }
 
+    my $dep_regex = qr{
+        ^\s*                            # Skip leading whitespace.
+        ($pkgname_regex)                # Package name.
+        (?:                             # Start of optional part.
+            :                           # Colon for architecture.
+            ([a-zA-Z0-9][a-zA-Z0-9-]*)  # Architecture name.
+        )?                              # End of optional part.
+        (?:                             # Start of optional part.
+            \s* \(                      # Open parenthesis for version part.
+            \s* (<<|<=|=|>=|>>|[<>])    # Relation part.
+            \s* ([^\)\s]+)              # Do not attempt to parse version.
+            \s* \)                      # Closing parenthesis.
+        )?                              # End of optional part.
+        (?:                             # Start of optional architecture.
+            \s* \[                      # Open bracket for architecture.
+            \s* ([^\]]+)                # Do not parse architectures now.
+            \s* \]                      # Closing bracket.
+        )?                              # End of optional architecture.
+        (
+            (?:                         # Start of optional restriction.
+                \s* <                   # Open bracket for restriction.
+                \s* [^>]+               # Do not parse restrictions now.
+                \s* >                   # Closing bracket.
+            )+
+        )?                              # End of optional restriction.
+        \s*$                            # Trailing spaces at end.
+    }x;
+
     ## no critic (RegularExpressions::ProhibitCaptureWithoutTest)
-    return if not $dep =~
-           m{^\s*                           # skip leading whitespace
-              ($pkgname_re)                 # package name
-              (?:                           # start of optional part
-                :                           # colon for architecture
-                ([a-zA-Z0-9][a-zA-Z0-9-]*)  # architecture name
-              )?                            # end of optional part
-              (?:                           # start of optional part
-                \s* \(                      # open parenthesis for version part
-                \s* (<<|<=|=|>=|>>|[<>])    # relation part
-                \s* ([^\)\s]+)              # do not attempt to parse version
-                \s* \)                      # closing parenthesis
-              )?                            # end of optional part
-              (?:                           # start of optional architecture
-                \s* \[                      # open bracket for architecture
-                \s* ([^\]]+)                # don't parse architectures now
-                \s* \]                      # closing bracket
-              )?                            # end of optional architecture
-              (
-                (?:                         # start of optional restriction
-                \s* <                       # open bracket for restriction
-                \s* [^>]+                   # do not parse restrictions now
-                \s* >                       # closing bracket
-                )+
-              )?                            # end of optional restriction
-              \s*$                          # trailing spaces at end
-            }x;
+    return if not $dep =~ $dep_regex;
     if (defined $2) {
         return if $2 eq 'native' and not $self->{build_dep};
         $self->{archqual} = $2;
@@ -252,7 +255,7 @@ sub output {
         $res .= ':' . $self->{archqual};
     }
     if (defined $self->{relation}) {
-        $res .= ' (' . $self->{relation} . ' ' . $self->{version} .  ')';
+        $res .= ' (' . $self->{relation} . ' ' . $self->{version} . ')';
     }
     if (defined $self->{arches}) {
         $res .= ' [' . join(' ', @{$self->{arches}}) . ']';
@@ -277,7 +280,7 @@ Save the dependency into the given $filename.
 # _arch_is_superset(\@p, \@q)
 #
 # Returns true if the arch list @p is a superset of arch list @q.
-# The arguments can also be undef in case there's no explicit architecture
+# The arguments can also be undef in case there is no explicit architecture
 # restriction.
 sub _arch_is_superset {
     my ($p, $q) = @_;
@@ -285,16 +288,16 @@ sub _arch_is_superset {
     my $q_arch_neg = defined $q and $q->[0] =~ /^!/;
 
     if (not defined $p) {
-        # If "p" has no arches, it is a superset of q and we should fall through
-        # to the version check.
+        # If $p has no arches, it is a superset of $q and we should fall
+        # through to the version check.
         return 1;
     } elsif (not defined $q) {
-        # If q has no arches, it is a superset of p and there are no useful
+        # If $q has no arches, it is a superset of $p and there are no useful
         # implications.
         return 0;
     } elsif (not $p_arch_neg and not $q_arch_neg) {
-        # Both have arches.  If neither are negated, we know nothing useful
-        # unless q is a subset of p.
+        # Both have arches. If neither are negated, we know nothing useful
+        # unless $q is a subset of $p.
 
         my %p_arches = map { $_ => 1 } @{$p};
         my $subset = 1;
@@ -303,8 +306,8 @@ sub _arch_is_superset {
         }
         return 0 unless $subset;
     } elsif ($p_arch_neg and $q_arch_neg) {
-        # If both are negated, we know nothing useful unless p is a subset of
-        # q (and therefore has fewer things excluded, and therefore is more
+        # If both are negated, we know nothing useful unless $p is a subset of
+        # $q (and therefore has fewer things excluded, and therefore is more
         # general).
 
         my %q_arches = map { $_ => 1 } @{$q};
@@ -314,12 +317,12 @@ sub _arch_is_superset {
         }
         return 0 unless $subset;
     } elsif (not $p_arch_neg and $q_arch_neg) {
-        # If q is negated and p isn't, we'd need to know the full list of
-        # arches to know if there's any relationship, so bail.
+        # If $q is negated and $p is not, we would need to know the full list
+        # of arches to know if there is any relationship, so bail.
         return 0;
     } elsif ($p_arch_neg and not $q_arch_neg) {
-        # If p is negated and q isn't, q is a subset of p if none of the
-        # negated arches in p are present in q.
+        # If $p is negated and $q is not, $q is a subset of $p if none of the
+        # negated arches in $p are present in $q.
 
         my %q_arches = map { $_ => 1 } @{$q};
         my $subset = 1;
@@ -355,29 +358,29 @@ sub _arch_qualifier_implies {
 #
 # Returns true if the restrictions $p and $q are compatible with the
 # implication $p -> $q, false otherwise.
-# NOTE: We don't try to be very clever here, so we may conservatively
+# Note: We do not try to be very clever here, so we may conservatively
 # return false when there is an implication.
 sub _restrictions_imply {
     my ($p, $q) = @_;
 
     if (not defined $p) {
-       return 1;
+        return 1;
     } elsif (not defined $q) {
-       return 0;
+        return 0;
     } else {
-       # Check whether set difference is empty.
-       my %restr;
+        # Check whether set difference is empty.
+        my %restr;
 
-       for my $restrlist (@{$q}) {
-           my $reststr = join ' ', sort @{$restrlist};
-           $restr{$reststr} = 1;
-       }
-       for my $restrlist (@{$p}) {
-           my $reststr = join ' ', sort @{$restrlist};
-           delete $restr{$reststr};
-       }
+        for my $restrlist (@{$q}) {
+            my $reststr = join ' ', sort @{$restrlist};
+            $restr{$reststr} = 1;
+        }
+        for my $restrlist (@{$p}) {
+            my $reststr = join ' ', sort @{$restrlist};
+            delete $restr{$reststr};
+        }
 
-       return keys %restr == 0;
+        return keys %restr == 0;
     }
 }
 
@@ -393,32 +396,32 @@ sub implies {
     my ($self, $o) = @_;
 
     if ($o->isa('Dpkg::Deps::Simple')) {
-        # An implication is only possible on the same package
+        # An implication is only possible on the same package.
         return if $self->{package} ne $o->{package};
 
         # Our architecture set must be a superset of the architectures for
-        # o, otherwise we can't conclude anything.
+        # $o, otherwise we cannot conclude anything.
         return unless _arch_is_superset($self->{arches}, $o->{arches});
 
-        # The arch qualifier must not forbid an implication
+        # The arch qualifier must not forbid an implication.
         return unless _arch_qualifier_implies($self->{archqual},
                                               $o->{archqual});
 
-        # Our restrictions must imply the restrictions for o
+        # Our restrictions must imply the restrictions for $o.
         return unless _restrictions_imply($self->{restrictions},
                                           $o->{restrictions});
 
-        # If o has no version clause, then our dependency is stronger
+        # If $o has no version clause, then our dependency is stronger.
         return 1 if not defined $o->{relation};
-        # If o has a version clause, we must also have one, otherwise there
-        # can't be an implication
+        # If $o has a version clause, we must also have one, otherwise there
+        # cannot be an implication.
         return if not defined $self->{relation};
 
         return Dpkg::Deps::deps_eval_implication($self->{relation},
                 $self->{version}, $o->{relation}, $o->{version});
     } elsif ($o->isa('Dpkg::Deps::AND')) {
-        # TRUE: Need to imply all individual elements
-        # FALSE: Need to NOT imply at least one individual element
+        # true: Need to imply all individual elements.
+        # false: Need to not imply at least one individual element.
         my $res = 1;
         foreach my $dep ($o->get_deps()) {
             my $implication = $self->implies($dep);
@@ -429,21 +432,19 @@ sub implies {
         }
         return $res;
     } elsif ($o->isa('Dpkg::Deps::OR')) {
-        # TRUE: Need to imply at least one individual element
-        # FALSE: Need to not apply all individual elements
-        # UNDEF: The rest
+        # true: Need to imply at least one individual element.
+        # false: Need to not apply all individual elements.
+        # undef: The rest.
         my $res = undef;
         foreach my $dep ($o->get_deps()) {
             my $implication = $self->implies($dep);
             if (defined $implication) {
                 if (not defined $res) {
                     $res = $implication;
+                } elsif ($implication) {
+                    $res = 1;
                 } else {
-                    if ($implication) {
-                        $res = 1;
-                    } else {
-                        $res = 0;
-                    }
+                    $res = 0;
                 }
                 last if defined $res and $res == 1;
             }
@@ -475,7 +476,7 @@ This method is a no-op for this object.
 =cut
 
 sub sort {
-    # Nothing to sort
+    # Nothing to sort.
 }
 
 =item $dep->arch_is_concerned($arch)
@@ -487,8 +488,10 @@ Returns true if the dependency applies to the indicated architecture.
 sub arch_is_concerned {
     my ($self, $host_arch) = @_;
 
-    return 0 if not defined $self->{package}; # Empty dep
-    return 1 if not defined $self->{arches};  # Dep without arch spec
+    # Empty dep.
+    return 0 if not defined $self->{package};
+    # Dep without arch spec.
+    return 1 if not defined $self->{arches};
 
     return debarch_is_concerned($host_arch, @{$self->{arches}});
 }
@@ -537,8 +540,10 @@ Returns true if the dependency applies to the indicated profile.
 sub profile_is_concerned {
     my ($self, $build_profiles) = @_;
 
-    return 0 if not defined $self->{package}; # Empty dep
-    return 1 if not defined $self->{restrictions}; # Dep without restrictions
+    # Empty dep.
+    return 0 if not defined $self->{package};
+    # Dep without restrictions.
+    return 1 if not defined $self->{restrictions};
     return evaluate_restriction_formula($self->{restrictions}, $build_profiles);
 }
 
@@ -622,7 +627,7 @@ sub merge_union {
     return 0 if defined $self->{arches} or defined $o->{arches};
 
     if (not defined $o->{relation} and defined $self->{relation}) {
-        # Union is the non-versioned dependency
+        # Union is the non-versioned dependency.
         $self->{relation} = undef;
         $self->{version} = undef;
         return 1;
